"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _Util_client;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Util = void 0;
const Constants_1 = require("./Constants");
const axios_1 = __importDefault(require("axios"));
const crypto_1 = __importDefault(require("crypto"));
/**
 * Utility class used by the Client
 */
class Util {
    /**
     * @param client Client attached to the Utility class
     */
    constructor(client) {
        /**
         * Private member for the Client instance
         */
        _Util_client.set(this, void 0);
        __classPrivateFieldSet(this, _Util_client, client, "f");
    }
    /**
     * Asserts a claim to be true
     * @param claim The claim to check
     * @param error The message to send if the claim evaluted to false
     * @internal
     */
    assert(claim, error) {
        if (!claim) {
            if (typeof error === "string") {
                throw new Error(error);
            }
            else if (error instanceof Error) {
                throw error;
            }
        }
        return true;
    }
    /**
     * Generates a basic token
     * @param opts Options to create the token with
     * @param opts.clientId The client id to use for the token
     * @param opts.clientSecret The client secret to use for the token
     * @param opts.length The length of the token to create
     * @returns A basic auth token
     */
    createBasicAuth(opts) {
        const { length, clientId, clientSecret } = Object.assign(Constants_1.DefaultBasicAuthConfig, opts);
        const uuid = crypto_1.default.randomUUID().replace(/\-/g, "");
        let hex;
        switch (length) {
            case 112:
                hex = uuid.toUpperCase();
                break;
            case 156:
                hex = crypto_1.default
                    .createHash("sha256")
                    .update(uuid)
                    .digest("hex")
                    .toUpperCase();
                break;
            default:
                throw TypeError("Invalid token length. Expected 112 | 156");
        }
        const a = hex + `_${clientId}:`;
        const b = hex + `:${clientId}:${clientSecret}`;
        const c = crypto_1.default.createHash("sha1").update(b).digest("hex");
        return Buffer.from(a + c).toString("base64");
    }
    /**
     * Validates that something is an Axios Error
     * @param item Item to validate
     * @internal
     */
    isAxiosError(item) {
        if (!axios_1.default.isAxiosError(item) || !item.response)
            return false;
        return true;
    }
    /**
     * Validates that an object matches the RESTAPIErrorResponse schema
     * @param error Error to validate
     * @returns error is RESTAPIErrorResponse
     * @internal
     */
    isAPIError(error) {
        if (!this.hasProperty(error, "error"))
            return false;
        if (!this.hasProperty(error, "error_description"))
            return false;
        if (!this.hasProperty(error, "status"))
            return false;
        if (typeof error.error !== "string")
            return false;
        if (typeof error.error_description !== "string")
            return false;
        if (typeof error.status !== "number")
            return false;
        return true;
    }
    /**
     * Certain requests have a header for 'if-modified-since' which uses the current timestamp
     * @param date Custom date
     * @returns GMT timestamp of the date
     * @internal
     */
    ifModifiedSince(date) {
        date ?? (date = new Date());
        return date.toUTCString();
    }
    /**
     * Paginates through items in a REST API endpoint.
     * @param url The url to get
     * @param key The key to get the array of values from
     * @param params The params to pass to the url
     * @param full Whether to yield the full data or just the items (default false)
     * @yields The value of the key
     * @internal
     */
    async *paginate(url, key, params, full = false) {
        // Verify config
        if (!url)
            throw new Error("paginate: url is required");
        if (!key)
            throw new Error("paginate: key is required");
        // Set method
        let method = "GET";
        if (url.toLowerCase() === "feeds/collective") {
            method = "POST";
        }
        let hasNext;
        const data = new URLSearchParams();
        // Set params
        for (const key in params) {
            data.set(key, `${params[key]}`);
        }
        // Request config
        const config = method === "GET" ? { method, url, params: data } : { method, url, data };
        do {
            const response = await __classPrivateFieldGet(this, _Util_client, "f").instance.request(config);
            const items = response.data?.data?.[key]?.items;
            if (!Array.isArray(items))
                throw new Error(`paginate: items is not an array`);
            // Set next param
            hasNext = response.data?.data?.[key]?.paging?.hasNext ?? false;
            if (hasNext) {
                data.set("next", response.data.data[key].paging.cursors.next);
            }
            // Yield the full data if wanted
            if (full) {
                //console.log("yield full");
                yield response.data;
                continue;
            }
            for (const item of items) {
                //console.log("yield item");
                yield item;
            }
        } while (hasNext);
    }
    /**
     * Sleeps the program for the specified amount of milliseconds
     * @param ms The number of milliseconds to wait
     * @returns A promise that resolves after the given amount of time
     */
    async sleep(ms) {
        return new Promise((resolve) => setTimeout(resolve, ms));
    }
    /**
     * Converts data into a json like string
     * @param data Data to convert to JSON
     * @returns JSON data
     */
    toJSON(data) {
        return JSON.stringify(data, null, 2);
    }
    /**
     * Type guard to validate key existance on an object
     * @param obj Object to check key for
     * @param prop Property to check is defined
     * @returns `obj` has property `prop`
     * @internal
     */
    hasProperty(obj, prop) {
        return Object.prototype.hasOwnProperty.call(obj, prop);
    }
    /**
     * Client headers
     * @returns AxiosRequestHeaders
     */
    get headers() {
        return __classPrivateFieldGet(this, _Util_client, "f").instance.defaults.headers.common;
    }
    /**
     * Updates client headers
     */
    set headers(headers) {
        Object.assign(__classPrivateFieldGet(this, _Util_client, "f").instance.defaults.headers.common, headers);
    }
}
exports.Util = Util;
_Util_client = new WeakMap();
exports.default = Util;
//# sourceMappingURL=Util.js.map