import { APIClientUser } from "@ifunny/ifunny-api-types";
import { BaseClient } from "./BaseClient";
import { ClientOptions } from "./BaseClient";
import { ContentManager } from "../managers/ContentManager";
import { FeedManager } from "../managers/FeedManager";
import { If } from "../utils/Types";
import { UserManager } from "../managers/UserManager";
import { Util } from "../utils/Util";
import { AppManager } from "../managers/AppManager";
import User from "../structures/User";
/**
 * Method params for Client#signUp()
 */
export interface SignUpOptions {
    /**
     * The nick to use when creating the account
     */
    nick: string;
    /**
     * Email / Username for the account
     */
    email: string;
    /**
     * Password for the account
     */
    password: string;
    /**
     * Should the Client sign up for mail offers? Never set this to true you weirdo
     */
    acceptMailing?: boolean;
    /**
     * Should the Client login to the account after creating it?
     */
    login?: boolean;
}
/**
 * Client for the iFunny API.
 * @extends BaseClient
 * @template Authorized
 */
export declare class Client<Authorized extends boolean = boolean> extends BaseClient {
    #private;
    /**
     * @param config The config for the Client
     * @param payload Payload for the client if applicable
     */
    constructor(config?: ClientOptions, payload?: Partial<APIClientUser>);
    /**
     * Utility class for the client
     */
    get util(): Util;
    /**
     * Basic token used by the client.\
     * If not set in constructor, one was generated
     */
    get basic(): string;
    /**
     * Sets the basic token for the Client. Must be a string
     */
    set basic(value: string);
    /**
     * Prime the Client's basic token
     * @returns The Client's basic token
     */
    primeBasic(): Promise<string>;
    /**
     * Primes a basic token without updating the Client config. This takes ~15 seconds
     * @param basic Basic token to prime
     * @returns Basic token being primed
     */
    primeBasic(basic: string): Promise<string>;
    /**
     * The bearer token for the client
     */
    get bearer(): If<Authorized, string, null>;
    /**
     * Sets the bearer token for the client. If set to null, the client will not use a bearer token.
     */
    set bearer(value: string | null);
    /**
     * The authorization string used for requests
     */
    get authorization(): string;
    /**
     * Is the client using a bearer token to make requests?
     */
    isAuthorized(): this is Client<true>;
    /**
     * Updates the Client's payload
     * @returns The Client instance
     */
    fetch(): Promise<this>;
    /**
     * The client's user manager
     */
    get users(): UserManager;
    /**
     * The content feeds for the Client
     */
    get feeds(): FeedManager;
    /**
     * The Client's Content manager
     */
    get content(): ContentManager;
    get app(): AppManager;
    user(): Promise<If<Authorized, User, null>>;
    /**
     * Logs in using the stored bearer token.\
     * Does NOT Generate a new bearer token
     */
    login(): Promise<this>;
    /**
     * Logs the Client in an generates a new Bearer token.\
     * If a bearer token is stored, these are ignored
     * @param email The email to log in with
     * @param password The password to log in with
     */
    login(email: string, password: string): Promise<this>;
    /**
     * Signs up for an iFunny account.
     * @param options.nick The nick of the account to sign up with. Checks if the nick is taken.
     * @param options.email The email to sign up with. Checks if the email is taken.
     * @param options.password The password to sign up with
     * @param options.acceptMailOffers Should the Client sign up for the mailing list. Never set this to true.
     * @param options.login Should the client login after sign up? (Default: true)
     * @returns The instance of the client
     */
    signUp(options: SignUpOptions): Promise<this>;
    protected get<K extends keyof APIClientUser>(key: K): If<Authorized, APIClientUser[K], null>;
    get about(): If<Authorized, string, null>;
    get id(): If<Authorized, string, null>;
}
export default Client;
//# sourceMappingURL=Client.d.ts.map